@@ -26,6 +26,7 @@ __PACKAGE__->mk_accessors(
     insecure
     logger
     order
+    no_passkey_warning
     print_error
     profile
     raise_error
@@ -45,6 +46,7 @@ __PACKAGE__->mk_accessors(
 
 use Carp;
 use Config::Tiny;
+use Cwd;
 use Data::Dumper;
 use Date::Format;
 use English qw {-no_match_vars};
@@ -52,7 +54,7 @@ use Exporter;
 use File::chdir;
 use File::HomeDir;
 use HTTP::Request;
-use JSON::PP;
+use JSON::PP qw{decode_json encode_json};
 use LWP::UserAgent;
 use MIME::Base64;
 use POSIX::strptime qw/strptime/;
@@ -60,7 +62,7 @@ use Time::Local;
 use Scalar::Util qw/reftype/;
 use List::Util qw/pairs any/;
 
-use constant { ## no critic (ValuesAndExpressions::ProhibitConstantPragma, NamingConventions::Capitalization)
+use constant {  ## no critic (ValuesAndExpressions::ProhibitConstantPragma, NamingConventions::Capitalization)
   AWS_AVAILABILITY_ZONE_URL =>
     q{latest/meta-data/placement/availability-zone},
   AWS_CONTAINER_CREDENTIALS_URL    => q{http://169.254.170.2/},
@@ -95,14 +97,14 @@ use constant { ## no critic (ValuesAndExpressions::ProhibitConstantPragma, Namin
   X_AMZ_SSO_BEARER_TOKEN        => ':x-amz-sso_bearer_token',
 };
 
-our $VERSION = '1.1.11'; ## no critic (ValuesAndExpressions::RequireInterpolationOfMetachars)
+our $VERSION = '1.1.15'; ## no critic (ValuesAndExpressions::RequireInterpolationOfMetachars)
 
 our @EXPORT_OK
   = qw{ create_passkey set_sso_credentials get_role_credentials };
 
 # we only log at debug level, create a default logger
 {
-  no strict 'refs'; ## no critic (TestingAndDebugging::ProhibitNoStrict)
+  no strict 'refs';  ## no critic (TestingAndDebugging::ProhibitNoStrict)
 
   *{'Amazon::Credentials::Logger::debug'} = sub {
     my ( $self, @message ) = @_;
@@ -135,7 +137,31 @@ sub new {
 
   $self->_init_logger;
 
-  $self->_init_encryption($passkey);
+  # note that multiple instances of the Amazon::Credentials will share
+  # the same passkey...this is probably a bug
+  if ( $self->get_passkey && $passkey ) {
+    if ( !$self->get_no_passkey_warning ) {
+      carp <<'WARNING';
+WARNING: the encryption passkey has already been set. Resetting the
+passkey will require that you restore the original passkey if you are
+using more than one instance of Amazon::Credentials
+WARNING
+    }
+
+    $self->_init_encryption($passkey);
+  }
+  elsif ( $self->get_passkey ) {
+    if ( !$self->get_no_passkey_warning ) {
+      carp <<'WARNING';
+WARNING: the encryption passkey has already been set. Using the previous passkey.
+WARNING
+    }
+
+    $self->_init_encryption( $self->get_passkey );
+  }
+  else {
+    $self->_init_encryption($passkey);
+  }
 
   if ( $self->get_insecure ) {
     $self->get_logger->debug( "!! CAUTION !!\n"
@@ -147,6 +173,7 @@ sub new {
   if ( $self->get_sso_role_name && $self->get_sso_account_id ) {
     my $region = $self->get_sso_region || $self->get_region;
 
+    # this just sets the environmet
     set_sso_credentials( $self->get_sso_role_name, $self->get_sso_account_id,
       $region );
 
@@ -167,6 +194,9 @@ sub new {
     } ## end if ( !$self->get_cache)
   } ## end if ( !$options->{'aws_secret_access_key'...})
   else {
+    $self->get_logger->debug( 'setting credentials from options ',
+      Dumper( [$options] ) );
+
     $self->set_credentials($options);
   } ## end else [ if ( !$options->{'aws_secret_access_key'...})]
 
@@ -413,6 +443,8 @@ sub get_creds_from_env {
     @{$creds}{@cred_keys} = ( 'ENV', @ENV{@env_keys} );
   } ## end if ( $ENV{'AWS_ACCESS_KEY_ID'...})
 
+  @ENV{@cred_keys} = (EMPTY) x 4;
+
   return $creds;
 } ## end sub get_creds_from_env
 
@@ -691,7 +723,7 @@ sub get_creds_from_container {
       # if not 200, then get out of Dodge
       if ( $rsp->is_success ) {
 
-        my $this = JSON::PP->new->utf8->decode( $rsp->content );
+        my $this = decode_json( $rsp->content );
 
         @{$creds}{
           qw/source container aws_access_key_id aws_secret_access_key token expiration/
@@ -964,8 +996,8 @@ sub get_token {
     $self->set_credentials;
   } ## end if ( !$self->get__session_token...)
 
-  my $token
-    = $self->decrypt( $self->get__session_token, $self->_fetch_passkey );
+  my $passkey = $self->_fetch_passkey;
+  my $token   = $self->decrypt( $self->get__session_token, $passkey );
 
   if ( !$self->get_cache ) {
     $self->set__session_token(undef);
@@ -1113,7 +1145,7 @@ sub set_sso_credentials {
       }
     ) {
 
-      $ENV{ $p->[1] } = sprintf '%s', $credentials->{ $p->[0] }; ## no critic (Variables::RequireLocalizedPunctuationVars)
+      $ENV{ $p->[1] } = sprintf '%s', $credentials->{ $p->[0] };  ## no critic (Variables::RequireLocalizedPunctuationVars)
     }
 
   }
@@ -1150,15 +1182,36 @@ sub get_role_credentials {
 
   my $rsp = $ua->get($url);
 
-  if ( $rsp && !$rsp->is_success ) {
-    croak $rsp->status_line . q{:} . Dumper( $rsp->content );
-  }
-  elsif ( !$rsp ) {
-    croak "no response from get($url)\n";
+  if ( $ENV{DEBUG} ) {
+    print {*STDERR} Dumper( [ 'response:', $rsp ] );
   }
 
-  return
-    eval { return JSON::PP->new->decode( $rsp->content )->{roleCredentials}; };
+  my $content;
+
+  if ( $rsp && $rsp->content_type eq 'application/json' ) {
+    $content = eval { return decode_json( $rsp->content ); };
+
+    croak sprintf "could not decode response from GetRoleCredentials\n%s",
+      $EVAL_ERROR
+      if !$content;
+  }
+
+  if ( $rsp && !$rsp->is_success ) {
+    if ( $content
+      && ref $content
+      && $content->{message} ) {
+      croak sprintf "Status: %s\n%s", $rsp->status_line, $content->{message};
+    }
+    else {
+      croak sprintf "Status: %s\n", $rsp->status_line,
+        Dumper( $rsp->content );
+    }
+  }
+  elsif ( !$rsp ) {
+    croak "no response from get($url)";
+  }
+
+  return $content->{roleCredentials};
 }
 
 # +-----------------+
@@ -1223,14 +1276,23 @@ sub _set_defaults {
   return $self;
 } ## end sub _set_defaults
 
+########################################################################
+# note that $passkey is a class variable and as such, once initialized
+# in the fashion below, will persist for all instances of
+# Amazon::Credentials
+########################################################################
 {
   my $passkey;
 
+########################################################################
   sub get_passkey {
+########################################################################
     return $passkey;
   }
 
+########################################################################
   sub set_passkey {
+########################################################################
     my ( $self, $key ) = @_;
 
     $passkey = $key;
@@ -1364,6 +1426,7 @@ sub _get_access_token {
   my ($home) = @_;
 
   require File::Find;
+
   File::Find->import('find');
 
   $home //= $ENV{HOME};
@@ -1374,6 +1437,8 @@ sub _get_access_token {
 
   my $access_token;
 
+  my $cwd = getcwd;
+
   eval {
 
     find(
@@ -1385,7 +1450,7 @@ sub _get_access_token {
         open my $fh, '<', $File::Find::name
           or croak 'could not open ' . $File::Find::name;
 
-        my $json = eval { JSON::PP->new->decode(<$fh>) };
+        my $json = eval { decode_json(<$fh>) };
 
         close $fh;
 
@@ -1400,6 +1465,8 @@ sub _get_access_token {
     );
   };
 
+  CORE::chdir $cwd;
+
   return $access_token;
 }
 
@@ -1506,7 +1573,7 @@ Amazon::Credentials - fetch Amazon credentials from file, environment or role
 
 CLI
 
- amazon-credentials.sh --help
+ amazon-credentials --help
 
 =head1 DESCRIPTION
 
@@ -1549,7 +1616,7 @@ or pass your SSO role name and account ID...
  
 =head1 VERSION
 
-This document reverse to verion 1.1.11 of
+This document reverse to verion 1.1.15 of
 C<Amazon::Credentials>.
 
 =head1 METHODS AND SUBROUTINES
@@ -1680,6 +1747,24 @@ conform to these may still be exposed. Caution is advised.>
 Pass in your own logger that has a C<debug()> method.  Otherwise the
 default logger will output debug messages to STDERR.
 
+=item no_passkey_warning
+
+Boolean that indicates whether warning messages about passkey usage
+should be supressed.
+
+If you attempt to reset the passkey or if you instantiate a second
+instance of Amazon::Credentials, the constructor will issue warnings.
+
+Resetting a passkey means that you previous version of
+Amazon::Credentials will no longer be able to decrypt credentials
+unless you restore the original passkey.
+
+If you instantiate another version of Amazon::Credentials without
+resetting the passkey, the new instance will use the old value for the
+passkey. This is by design.
+
+default: false
+
 =item order
 
 An array reference containing tokens that specifies the order in which the class will
@@ -1703,7 +1788,6 @@ new passkey method.
  $credentials->set_passkey(\&new_passkey_provider);
  $credentials->reset_credentials(1);
 
-
 =item print_error
 
 Whether to print the error if no credenials are found. C<raise_error>
@@ -1753,7 +1837,9 @@ expires.
 =item timeout
 
 When looking for credentials in metadata URLs, this parameter
-specifies the timeout value for C<LWP>.  The default is 3 seconds.
+specifies the timeout value for C<LWP>.
+
+default: 3s
 
 =item user_agent
 
@@ -2151,27 +2237,122 @@ initialization and use that to encrypt and decrypt the
 credentials. Obviously the passkey must be available for
 C<Amazon::Credentials> to decrypt the keys, however it is B<NOT>
 stored in the blessed hash reference that stores other data used by
-the class. To avoid having the class know about your passkey at all, pass a
+the class. Instead the passkey is a class variable and will be
+initialized once for all instances of C<Amazon::Credentials> your
+script uses.
+
+If you plan on using multiple instances of C<Amazon::Credentials> B<and>
+you are passing in your own passkeys, then you'll need to reset the
+passkey for each use of the credentials. See the example below in the
+L</Using Multiple Instances of Amazon::Credentials> section.
+
+To avoid having the class know about your passkey at all, pass a
 reference to a subroutine that will provide the passkey for encryption
 and decryption. You can even use the same passkey generator that is
 used by C<Amazon::Credentials> (C<create_passkey>).
 
-The point here is to avoid storing your passkey the same object as the
-credentials to minimize the likelihood of exposing your credentials or
-your methods for encryption in logs...better but not perfect. It's
-still may be possible to expose your passkey and your credentials if
-you are not careful.
+The point here is to avoid storing your passkey in the same object as
+the credentials to minimize the likelihood of exposing your
+credentials or your methods for encryption in logs...better but not
+perfect. It's still may be possible to expose your passkey and your
+credentials if you are not careful.
 
  use Amazon::Credentials qw{ create_passkey };
 
  my $passkey = create_passkey();
 
- my $credentials = Amazon::Credentials( passkey => sub { return $passkey } );
+ my $credentials = Amazon::Credentials->new(
+      passkey => sub { return caller(0) eq 'Amazon::Credentials' && $passkey },
+  );
 
 A more secure approach would be for your subroutine to retrieve a
 passkey from a source other than your own program and B<never> store
 the passkey inside your program.
 
+=item Using Multiple Instances of Amazon::Credentials
+
+You may at times need to assume a role using initial credentials. In
+this case you can use multiple instances of
+C<Amazon::Credentials>. Let's suppose that you have logged in with
+your SSO credentials but your script must assume a role in another
+account to perform some action.
+
+ # 1. retrieve SSO credentials
+ my $sso_credentials = Amazon::Credentials->new(
+   sso_role_name  => 'developer',
+   sso_account_id => '01234567890'
+ );
+
+ # 2. assume a role in another account
+ my $role_arn = 'arn:aws:iam::09876543210:role/Route53AccountAccessRole';
+ my $role_session_name = "route53-role-$PID";
+ 
+ # using the SSO credentials which presumably allow you to assume the role...
+ my $sts = Amazon::API::STS->new( credentials => $sso_credentials );
+ 
+ my $assume_role_result = $sts->AssumeRole(
+   { RoleArn         => $role_arn,
+     RoleSessionName => $role_session_name,
+   }
+ );
+ 
+ my $assume_role_credentials = $assume_role_result->{AssumeRoleResult}->{Credentials};
+
+ # 3. create new credentials for assumed role
+ my $role_credentials = Amazon::Credentials->new(
+   aws_access_key_id     => $assume_role_credentials->{AccessKeyId},
+   aws_secret_access_key => $assume_role_credentials->{SecretAccessKey},
+   expiration            => $assume_role_credentials->{Expiration},
+   token                 => $assume_role_credentials->{SessionToken},
+  );
+ 
+ # 4. make a call to another API
+ my $rt53 = Amazon::API::Route53->new(
+   credentials => $role_credentials,
+  );
+ 
+ my $list_tags_for_resources_response = $rt53->ListTagsForResources(
+    { ResourceType => 'hostedzone',
+      ResourceIds  => \@zone_ids,
+    }
+  );
+
+As noted above, when you use multiple instances of
+C<Amazon::Credentials>, the I<same> passkey is used for encrypting
+credentials. To avoid this, you can pass a custom passkey when you
+instantiate the C<Amazon::Credentials> object, however, you will need
+to reset that passkey when you use that object.
+
+ use Amazon::Credentials qw{create_passkey};
+ 
+ my %passkey = (
+   sso  => create_passkey,
+   role => create_passkey,
+ );
+
+ my $sso_creds = sub { return $passkey{sso} };
+ my $role_creds = sub { return $passkey{role} };
+
+ my $sso_credentials = Amazon::Credentials->new(
+   sso_role_name  => 'developer',
+   sso_account_id => '01234567890'
+   passkey        => $sso_creds,
+ );
+
+ ... 
+
+ my $role_credentials = Amazon::Credentials->new(
+   aws_access_key_id     => $assume_role_credentials->{AccessKeyId},
+   aws_secret_access_key => $assume_role_credentials->{SecretAccessKey},
+   token                 => $assume_role_credentials->{SessionToken},
+   expiration            => $assume_role_credentials->{Expiration},
+   passkey               => $role_creds,
+ );
+
+...then later
+
+ $sso_credentials->set_passkey($sso_creds);
+
 =item Using a Custom Cipher
 
 As noted, the default C<Crypt::CBC> cipher is used for encrypting your
@@ -2203,7 +2384,7 @@ continue to provide the old passkey before you can reset the passkey.
 
  sub get_passkey {
    my ($regenerate) = shift;
-   
+
    return $regenerate ? create_passkey : $passkey;
  } 
  
